@@ -9,7 +9,34 @@ class NoFreeChannels(BaseException):
     """Raised when a connection has reached its channel limit"""
 
 
-class BadDeclaration(BaseException):
+class PermissionException(BaseException):
+    """
+    Generic permissions exception.
+
+    Args:
+        obj_type (str): The type of object being accessed that caused the
+            permission error. May be None if the cause is unknown.
+        description (object): The description of the object, if any. May be None.
+        reason (str): The reason the server gave for the permission error, if
+            any. If no reason is supplied by the server, this should be the best
+            guess for what caused the error.
+    """
+
+    def __init__(self, obj_type=None, description=None, reason=None):
+        self.obj_type = obj_type
+        self.description = description
+        self.reason = reason
+
+    def __str__(self):
+        return self.description
+
+    def __repr__(self):
+        return "PermissionException(obj_type={}, description={}, reason={})".format(
+            self.obj_type, self.description, self.reason
+        )
+
+
+class BadDeclaration(PermissionException):
     """
     Raised when declaring an object in AMQP fails.
 
@@ -20,10 +47,15 @@ class BadDeclaration(BaseException):
         reason (str): The reason the server gave for rejecting the declaration.
     """
 
-    def __init__(self, obj_type, description, reason):
-        self.obj_type = obj_type
-        self.description = description
-        self.reason = reason
+    def __str__(self):
+        return "Unable to declare the {} object ({}) because {}".format(
+            self.obj_type, self.description, self.reason
+        )
+
+    def __repr__(self):
+        return "BadDeclaration(obj_type={}, description={}, reason={})".format(
+            self.obj_type, self.description, self.reason
+        )
 
 
 class ConfigurationException(BaseException):
@@ -49,6 +81,9 @@ class PublishException(BaseException):
         super(PublishException, self).__init__(**kwargs)
         self.reason = reason
 
+    def __str__(self):
+        return str(self.reason)
+
 
 class PublishReturned(PublishException):
     """
@@ -59,6 +94,15 @@ class PublishReturned(PublishException):
     """
 
 
+class PublishTimeout(PublishException):
+    """
+    Raised when the message could not be published in the given timeout.
+
+    This means the message was either never delivered to the broker, or that it
+    was delivered, but never acknowledged by the broker.
+    """
+
+
 class ConnectionException(BaseException):
     """
     Raised if a general connection error occurred.
@@ -67,15 +111,24 @@ class ConnectionException(BaseException):
     message.
     """
 
-    def __init__(self, reason=None, **kwargs):
-        super(ConnectionException, self).__init__(**kwargs)
-        self.reason = reason
+    def __init__(self, *args, **kwargs):
+        super(ConnectionException, self).__init__(*args)
+        self.reason = kwargs.get("reason")
 
 
 class ConsumeException(BaseException):
     """Base class for exceptions related to consuming."""
 
 
+class ConsumerCanceled(ConsumeException):
+    """
+    Raised when the server has canceled the consumer.
+
+    This can happen when the queue the consumer is subscribed to is deleted,
+    or when the node the queue is located on fails.
+    """
+
+
 class Nack(ConsumeException):
     """
     Consumer callbacks should raise this to indicate they wish the message they
