@@ -16,4 +16,10 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 import os
 
+import crochet
+
 FIXTURES_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "fixtures/"))
+
+# Tests use pytest-twisted, which manages the Twisted reactor for us. This stops
+# crochet from starting the reactor in its own thread.
+crochet.no_setup()
