@@ -22,17 +22,23 @@ import unittest
 
 import mock
 import pika
-import pkg_resources
 import pytest
 from twisted.internet import defer, error
 
 from fedora_messaging import config
 from fedora_messaging.message import Message
-from fedora_messaging.exceptions import Nack, Drop, HaltConsumer
+from fedora_messaging.exceptions import (
+    Nack,
+    Drop,
+    HaltConsumer,
+    NoFreeChannels,
+    ConnectionException,
+)
 from fedora_messaging.twisted.protocol import (
     FedoraMessagingProtocol,
-    _pika_version,
+    FedoraMessagingProtocolV2,
     Consumer,
+    _add_timeout,
 )
 
 
@@ -94,6 +100,17 @@ class ProtocolTests(unittest.TestCase):
         self.factory = mock.Mock()
         self.protocol.factory = self.factory
 
+    def test_allocate_channel_no_more_channels(self):
+        """Assert a pika NoFreeChannels exception turns into a fedora_messaging exception."""
+        self.protocol.channel = mock.Mock(
+            name="channel", side_effect=pika.exceptions.NoFreeChannels()
+        )
+
+        d = self.protocol._allocate_channel()
+        d.addCallback(pytest.fail, "Expected a NoFreeChannels exception")
+        d.addErrback(lambda f: f.trap(NoFreeChannels))
+        pytest_twisted.blockon(d)
+
     @mock.patch(
         "fedora_messaging.twisted.protocol.uuid.uuid4", mock.Mock(return_value="tag1")
     )
@@ -104,6 +121,10 @@ class ProtocolTests(unittest.TestCase):
 
         def _check(consumer):
             assert self.protocol._running is True
+            consumer.channel.basic_qos.assert_called_with(
+                prefetch_count=config.conf["qos"]["prefetch_count"],
+                prefetch_size=config.conf["qos"]["prefetch_size"],
+            )
             consumer.channel.basic_consume.assert_called_once_with(
                 queue="my_queue", consumer_tag="tag1"
             )
@@ -142,6 +163,10 @@ class ProtocolTests(unittest.TestCase):
         func = mock.Mock()
 
         def _check(consumer):
+            consumer.channel.basic_qos.assert_called_with(
+                prefetch_count=config.conf["qos"]["prefetch_count"],
+                prefetch_size=config.conf["qos"]["prefetch_size"],
+            )
             consumer.channel.basic_consume.assert_called_once_with(
                 queue="my_queue", consumer_tag="tag1"
             )
@@ -196,19 +221,12 @@ class ProtocolTests(unittest.TestCase):
     def test_connection_ready(self):
         # Check the ready Deferred.
         def _check(_):
-            self.protocol._channel.basic_qos.assert_called_with(
-                prefetch_count=config.conf["qos"]["prefetch_count"],
-                prefetch_size=config.conf["qos"]["prefetch_size"],
-                all_channels=True,
-            )
+            self.protocol.channel.assert_called_once_with()
 
         d = self.protocol.ready
         d.addCallback(_check)
 
-        if _pika_version < pkg_resources.parse_version("1.0.0b1"):
-            self.protocol.connectionReady(None)
-        else:
-            self.protocol._on_connection_ready(None)
+        self.protocol._on_connection_ready(None)
 
         return pytest_twisted.blockon(d)
 
@@ -392,9 +410,8 @@ class ProtocolReadTests(unittest.TestCase):
             pika.exceptions.ChannelClosed(42, "testing"),
             pika.exceptions.ConsumerCancelled(),
             RuntimeError(),
+            pika.exceptions.ChannelClosedByClient(42, "testing"),
         ]
-        if _pika_version >= pkg_resources.parse_version("1.0.0b1"):
-            exceptions.append(pika.exceptions.ChannelClosedByClient(42, "testing"))
 
         deferreds = []
         for exc in exceptions:
@@ -524,3 +541,52 @@ class ProtocolOnMessageTests(unittest.TestCase):
 
         d.addCallback(_check)
         return pytest_twisted.blockon(d)
+
+
+class ProtocolV2Tests(unittest.TestCase):
+    """Unit tests for the ProtocolV2 class."""
+
+    def test_consume_connection_exception(self):
+        """If consuming fails due to a non-permission error, a ConnectionException happens."""
+        proto = FedoraMessagingProtocolV2(None)
+        mock_channel = mock.Mock()
+        mock_channel.basic_consume.side_effect = pika.exceptions.ChannelClosed(
+            400, "Bad Request!"
+        )
+        deferred_channel = defer.succeed(mock_channel)
+        proto._allocate_channel = mock.Mock(return_value=deferred_channel)
+
+        def check(failure):
+            assert isinstance(failure.value, ConnectionException)
+
+        d = proto.consume(lambda x: x, "test_queue")
+        d.addBoth(check)
+        return pytest_twisted.blockon(d)
+
+
+class AddTimeoutTests(unittest.TestCase):
+    """Unit tests for the _add_timeout helper function."""
+
+    def test_twisted12_timeout(self):
+        """Assert timeouts work for Twisted 12.2 (EL7)"""
+        d = defer.Deferred()
+        d.addTimeout = mock.Mock(side_effect=AttributeError())
+        _add_timeout(d, 0.1)
+
+        d.addCallback(self.fail, "Expected errback to be called")
+        d.addErrback(
+            lambda failure: self.assertIsInstance(failure.value, defer.CancelledError)
+        )
+
+        return pytest_twisted.blockon(d)
+
+    def test_twisted12_cancel_cancel_callback(self):
+        """Assert canceling the cancel call for Twisted 12.2 (EL7) works."""
+        d = defer.Deferred()
+        d.addTimeout = mock.Mock(side_effect=AttributeError())
+        d.cancel = mock.Mock()
+        with mock.patch("fedora_messaging.twisted.protocol.reactor") as mock_reactor:
+            _add_timeout(d, 1)
+            delayed_cancel = mock_reactor.callLater.return_value
+            d.callback(None)
+            delayed_cancel.cancel.assert_called_once_with()
